import React, { Component } from 'react';
import logo from './logo.svg';
import './App.css';

import { connect, Provider } from 'react-redux';
import {Router, Route, Link} from "react-router-dom";
import createHistory from "history/createBrowserHistory";
import { createStore, combineReducers } from 'redux';


function jsonPost(url, data)
{
    return new Promise((resolve, reject) => {
        var x = new XMLHttpRequest();   
        x.onerror = () => reject(new Error('jsonPost failed'))
        //x.setRequestHeader('Content-Type', 'application/json');
        x.open("POST", url, true);
        x.send(JSON.stringify(data))

        x.onreadystatechange = () => {
            if (x.readyState == XMLHttpRequest.DONE && x.status == 200){
                resolve(JSON.parse(x.responseText))
            }
            else if (x.status != 200){
                reject(new Error('status is not 200'))
            }
        }
    })
}


var addMessageReducer = function(state, action) {
    //console.log(action)
    if (state === undefined){
        return {};
    }
    if (action.type === 'addMessage') {
        return ({nick:action.nick, 
             message: action.message})
    }

    if (action.type === 'messageSent') {
        return ({})
    }
    return state;
}

var getMessagesReducer = function(state, action){
    if (state === undefined){
        return ({
            messageId: 0,
            messages: []
        })
    }
    if (action.type == 'getMessages'){
        console.log(state, action)
        return {
            messageId: action.data.nextMessageId,
            messages: [...state.messages,... action.data.data]
        }
    }
    return state;
}


const reducers = combineReducers({
    newMessage: addMessageReducer,
    messages: getMessagesReducer
})


var store = createStore(reducers);

store.subscribe(function(){
    var state = store.getState().newMessage;
    console.log(state)
    if (state.nick){
        jsonPost("http://students.a-level.com.ua:10012", {func: "addMessage", nick: state.nick, message: state.message, author: 'react'})
        .then(() => store.dispatch({type: 'messageSent'}))
    }
})

setInterval(function(){
    var messageId = store.getState().messages.messageId;
    jsonPost("http://students.a-level.com.ua:10012", {func: "getMessages", messageId, author: 'react'}) 
        .then((data) => store.dispatch({type: 'getMessages', data: data}))
},2000)

class ChatInput extends Component {
    constructor(){
        super();

        this.send = this.send.bind(this);
    }

    send(){
        store.dispatch({
            type: 'addMessage',
            nick: this.nick.value,
            message: this.message.value
        })
    }

    render(){
        console.log('RENDER CHAT INPUT', this.props.newMessage.nick)
        return (
            <div>
                <input type='text' placeholder='Nick' ref={(c) => this.nick = c} disabled={ !!this.props.newMessage.nick }/>
                <input type='text' placeholder='Message' ref={(c) => this.message = c} disabled={ !!this.props.newMessage.nick }/>
                <button onClick={this.send}>Send</button>
            </div>
        );
    }
}

class ChatLine extends Component{
    render(){
        return (
            <div><b>{this.props.nick}</b>:<span>{this.props.message}</span></div>
        );
    }
}

class Chat extends Component{
    render(){
        return (
        <div>
            { this.props.messages.messages.map((m, i) => <ChatLine nick={m.nick} message={m.message} key={i}/>) }
        </div>
        );
    }
}

const mapStateToProps = function(store) {
  return {
    newMessage: store.newMessage,
    messages: store.messages
  };
}

ChatInput = connect(mapStateToProps)(ChatInput)
Chat      = connect(mapStateToProps)(Chat)


class App extends Component {
  render() {
    return (
      <div className="App">
        <header className="App-header">
          <img src={logo} className="App-logo" alt="logo" />
          <h1 className="App-title">Welcome to React</h1>
        </header>
        <p className="App-intro">
          To get started, edit <code>src/App.js</code> and save to reload.
        </p>
        <Provider store={store}>
            <div>
                <ChatInput />
                <Chat />
            </div>
        </Provider>
      </div>
    );
  }
}

export default App;
